--[[-------------------------------------------------------------------------
	SmallerBlizzardRaidFrames
	Copyright (C) 2010  Morsker
	Please contact Morsker through PM on forums.wowace.com.

	Resizes Blizzard raid frames to be smaller squares.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.
---------------------------------------------------------------------------]]
local addonName = ...
local SBRF = LibStub("AceAddon-3.0"):NewAddon(addonName)
_G[addonName] = SBRF
local SavedVars_Name = addonName.."PerCharDB"

--------------
--  Locals  --
--------------

-- GLOBALS: _G
-- GLOBALS: InterfaceOptionsRaidFramePanelRaidFrameHeightSlider
-- GLOBALS: InterfaceOptionsRaidFramePanelRaidFrameWidthSlider
-- GLOBALS: CompactRaidFrameContainer
-- GLOBALS: CompactRaidFrameManagerContainerResizeFrame
-- GLOBALS: CompactRaidFrameManager_ResetContainerPosition
-- GLOBALS: GetCVar
-- GLOBALS: SetCVar

-- these declarations for UI config
local pairs = pairs
-- GLOBALS: GameTooltip
-- GLOBALS: GameTooltip_Hide
-- GLOBALS: CompactPartyFrame
-- GLOBALS: CreateFrame
-- GLOBALS: InterfaceOptionsRaidFramePanel
-- GLOBALS: InterfaceOptionsRaidFramePanelDisplayClassColor
-- GLOBALS: SBRF_CenterNames
-- GLOBALS: SBRF_CenterNamesText
-- GLOBALS: SBRF_VerticalHPText

local GameFontHighlightLarge = GameFontHighlightLarge
local GetScreenHeight = GetScreenHeight
local hooksecurefunc = hooksecurefunc
local Texture_SetSize = ActionButton1Icon.SetSize

local DBVERSION = 1
local db
local frame
local fixedFrames = {}

--((@)) local counts = {
--((@)) 	fixframe = 0,
--((@)) 	hook2 = 0,
--((@)) 	name = 0,
--((@)) 	force32 = 0,
--((@)) }
--((@)) _G.SBRF = SBRF
--((@)) SBRF.counts = counts

---------------------
--  Init / Events  --
---------------------

local function HideMe(self)
	self:Hide()
end

function SBRF.KeepDebuff32_32(debuffFrame)
	--((@)) counts.force32 = counts.force32 + 1
	Texture_SetSize(debuffFrame, 32, 32)
end

function SBRF.FixFrame(frame)
	--((@)) counts.fixframe = counts.fixframe + 1

	local name = frame.name
	name:SetFontObject(GameFontHighlightLarge)
	if db.centerNames then
		name:ClearAllPoints()
		name:SetPoint("CENTER")
	end

	Texture_SetSize(frame.roleIcon, 24, 24)
	
	local buffs = frame.buffFrames
	Texture_SetSize(buffs[1], 32, 32)
	Texture_SetSize(buffs[2], 32, 32)
	local debuffs = frame.debuffFrames
	local d_debuffs = frame.dispelDebuffFrames
	Texture_SetSize(d_debuffs[1], 32, 32)
	Texture_SetSize(d_debuffs[2], 32, 32)

	buffs[1]:ClearAllPoints()
	buffs[1]:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -3, 2)
	debuffs[1]:ClearAllPoints()
	debuffs[1]:SetPoint("TOPLEFT", frame, "TOPLEFT", 3, -2)
	
	if not fixedFrames[frame] then
		fixedFrames[frame] = true

		-- make debuffs 32x32
		hooksecurefunc(debuffs[1], "SetSize", SBRF.KeepDebuff32_32)
		hooksecurefunc(debuffs[2], "SetSize", SBRF.KeepDebuff32_32)
		
		-- vertical hp bars
		if db.verticalHP then
			frame.healthBar:SetOrientation("VERTICAL")
			frame.healthBar:SetRotatesTexture(true)
			frame.myHealPredictionBar:SetOrientation("VERTICAL")
			frame.myHealPredictionBar:SetRotatesTexture(true)
			frame.otherHealPredictionBar:SetOrientation("VERTICAL")
			frame.otherHealPredictionBar:SetRotatesTexture(true)
			-- frame.powerBar ? Making this vertical is more work, I'll see what people want first.
		end

		-- hide 3rd buffs
		buffs[3]:Hide()
		buffs[3]:SetScript("OnShow", HideMe)
		debuffs[3]:Hide()
		debuffs[3]:SetScript("OnShow", HideMe)
		d_debuffs[3]:Hide()
		d_debuffs[3]:SetScript("OnShow", HideMe)
	end
end

hooksecurefunc("DefaultCompactUnitFrameSetup", SBRF.FixFrame)
hooksecurefunc("CompactRaidFrameReservation_RegisterReservation", function(_, frame, _)
	--((@)) counts.hook2 = counts.hook2 + 1
	SBRF.FixFrame(frame)
end)

function SBRF.TruncateFrameName(frame)
	--((@)) counts.name = counts.name + 1
	local name = frame.name
	local nameText = name:GetText()
	if #nameText > 5 then
		name:SetText(nameText:sub(1,5))
	end
end

hooksecurefunc("CompactUnitFrame_UpdateName", SBRF.TruncateFrameName)

function SBRF.FixResizeFrame(frame)
	local resizeFrame = frame.containerResizeFrame
	resizeFrame:SetMaxResize(resizeFrame:GetWidth(), (GetScreenHeight()*2) - 90)
end

hooksecurefunc("CompactRaidFrameManager_UpdateContainerBounds", SBRF.FixResizeFrame)

function SBRF.FixMagnetism(manager)
	local point, relativeTo, relativePoint, xOffset, yOffset = manager.containerResizeFrame:GetPoint(1)
	if relativeTo == manager then
		CompactRaidFrameManager_ResetContainerPosition()
	end
end

hooksecurefunc("CompactRaidFrameManager_ResizeFrame_CheckMagnetism", SBRF.FixMagnetism)

function SBRF.Config_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetText(self.tooltip, nil, nil, nil, nil, 1);
end

function SBRF:OnInitialize()
	self.OnInitialize = nil

	_G[SavedVars_Name] = _G[SavedVars_Name] or { dbversion = DBVERSION }
	db = _G[SavedVars_Name]
	SBRF.db = db
	if db.centerNames == nil then db.centerNames = true end
	if db.verticalHP == nil then db.verticalHP = true end

	CompactRaidFrameContainer:SetScale(0.5)
	CompactRaidFrameManagerContainerResizeFrame:SetScale(0.5)
	CompactPartyFrame:SetScale(0.7)
	
	local check = CreateFrame("CheckButton", "SBRF_CenterNames", InterfaceOptionsRaidFramePanel, "InterfaceOptionsCheckButtonTemplate")
	check.tooltip = "Move name to center of frame."
	check:SetWidth(24)
	check:SetHeight(24)
	check:SetChecked(db.centerNames)
	check:SetPoint("TOPLEFT", InterfaceOptionsRaidFramePanelDisplayClassColor, "BOTTOMLEFT", 0, -8)
	check:SetScript("OnClick", function(self)
		db.centerNames = not db.centerNames
		self:SetChecked(db.centerNames)
		for frame,_ in pairs(fixedFrames) do
			local name = frame.name
			name:ClearAllPoints()
			if db.centerNames then
				name:SetPoint("CENTER")
			else
				name:SetPoint("TOPLEFT", frame.roleIcon, "TOPRIGHT", 0, -1);
				name:SetPoint("TOPRIGHT", -3, -3);
			end
		end
	end)
	check:SetScript("OnEnter", SBRF.Config_OnEnter)
	check:SetScript("OnLeave", GameTooltip_Hide)
	SBRF_CenterNamesText:SetText("Center Names")
	SBRF_CenterNamesText:SetPoint("LEFT", check, "RIGHT")
	
	local check = CreateFrame("CheckButton", "SBRF_VerticalHP", InterfaceOptionsRaidFramePanel, "InterfaceOptionsCheckButtonTemplate")
	check.tooltip = "Fill hp bottom-to-top, vertically."
	check:SetWidth(24)
	check:SetHeight(24)
	check:SetChecked(db.verticalHP)
	check:SetPoint("TOPLEFT", SBRF_CenterNames, "BOTTOMLEFT", 0, -8)
	check:SetScript("OnClick", function(self)
		db.verticalHP = not db.verticalHP
		self:SetChecked(db.verticalHP)

		local orientation = db.verticalHP and "VERTICAL" or "HORIZONTAL"
		for frame,_ in pairs(fixedFrames) do
			frame.healthBar:SetOrientation(orientation)
			frame.myHealPredictionBar:SetOrientation(orientation)
			frame.otherHealPredictionBar:SetOrientation(orientation)
		end
	end)
	check:SetScript("OnEnter", SBRF.Config_OnEnter)
	check:SetScript("OnLeave", GameTooltip_Hide)
	SBRF_VerticalHPText:SetText("Vertical HP")
	SBRF_VerticalHPText:SetPoint("LEFT", check, "RIGHT")
end

function SBRF:OnEnable()
end

function SBRF:OnDisable()
end
